﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;

namespace HeuristicLab.DataPreprocessing {
  public class SearchLogic : ISearchLogic {
    private readonly IPreprocessingData preprocessingData;

    public SearchLogic(IPreprocessingData thePreprocessingData) {
      preprocessingData = thePreprocessingData;
    }

    public IDictionary<string, IEnumerable<int>> GetMissingValueIndices() {
      var dic = new Dictionary<string, IEnumerable<int>>();
      foreach (string variableName in preprocessingData.VariableNames) {
        dic.Add(variableName, GetMissingValueIndices(preprocessingData.GetColumnIndex(variableName)));
      }
      return dic;
    }

    public bool IsMissingValue(int columnIndex, int rowIndex) {
      if (preprocessingData.IsType<double>(columnIndex)) {
        return double.IsNaN(preprocessingData.GetCell<double>(columnIndex, rowIndex));
      } else if (preprocessingData.IsType<string>(columnIndex)) {
        return string.IsNullOrEmpty(preprocessingData.GetCell<string>(columnIndex, rowIndex));
      } else if (preprocessingData.IsType<DateTime>(columnIndex)) {
        return preprocessingData.GetCell<DateTime>(columnIndex, rowIndex).Equals(DateTime.MinValue);
      } else {
        throw new ArgumentException("cell in column " + columnIndex + " and row index " + rowIndex + " contains a non supported type.");
      }
    }

    public IEnumerable<int> GetMissingValueIndices(int columnIndex) {
      if (preprocessingData.IsType<double>(columnIndex)) {
        return preprocessingData.GetValues<double>(columnIndex).Select((s, i) => new { i, s }).Where(t => double.IsNaN(t.s)).Select(t => t.i);
      } else if (preprocessingData.IsType<string>(columnIndex)) {
        return preprocessingData.GetValues<string>(columnIndex).Select((s, i) => new { i, s }).Where(t => string.IsNullOrEmpty(t.s)).Select(t => t.i);
      } else if (preprocessingData.IsType<DateTime>(columnIndex)) {
        return preprocessingData.GetValues<DateTime>(columnIndex).Select((s, i) => new { i, s }).Where(t => t.s.Equals(DateTime.MinValue)).Select(t => t.i);
      } else {
        throw new ArgumentException("column " + columnIndex + " contains a non supported type.");
      }
    }

  }
}
